@@ -20,8 +20,8 @@
  * SOFTWARE.
  */
 
-#include "ragel.h"
-#include "common.h"
+#include <libfsm/ragel.h>
+#include <libfsm/common.h>
 #include "inputdata.h"
 #include "parsedata.h"
 #include "load.h"
@@ -29,6 +29,8 @@
 #include "reducer.h"
 #include "version.h"
 #include "pcheck.h"
+#include <libfsm/dot.h>
+
 #include <colm/colm.h>
 
 #include <stdlib.h>
@@ -43,7 +45,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
+#if defined(HAVE_SYS_WAIT_H)
 #include <sys/wait.h>
+#endif
 
 #ifdef _WIN32
 #include <windows.h>
@@ -217,6 +221,10 @@ void InputData::verifyWritesHaveData()
 
 void InputData::writeOutput( InputItem *ii )
 {
+	/* If it is the first input item then check if we need to write the BOM. */
+	if ( ii->prev == 0 && utf8BomPresent )
+		*outStream << (uchar)0xEF << (uchar)0xBB << (uchar) 0xBF;
+
 	switch ( ii->type ) {
 		case InputItem::Write: {
 			CodeGenData *cgd = ii->pd->cgd;
@@ -249,12 +257,6 @@ void InputData::writeOutput( InputItem *ii )
 	}
 }
 
-void InputData::writeOutput()
-{
-	for ( InputItemList::Iter ii = inputItems; ii.lte(); ii++ )
-		writeOutput( ii );
-}
-
 void InputData::closeOutput()
 {
 	/* If writing to a file, delete the ostream, causing it to flush.
@@ -265,6 +267,13 @@ void InputData::closeOutput()
 	}
 }
 
+void InputData::writeDot( ostream &out )
+{
+	ParseData *pd = dotGenPd;
+	GraphvizDotGen dotGen( this, pd->fsmCtx, pd->sectionGraph, pd->sectionName, pd->machineId, out );
+	dotGen.write();
+}
+
 void InputData::processDot()
 {
 	/* Compiles the DOT machines. */
@@ -576,22 +585,21 @@ void InputData::usage()
 "   -S <spec>            FSM specification to output (for graphviz output)\n"
 "   -M <machine>         Machine definition/instantiation to output (for\n"
 "                        graphviz output)\n"
-"host language:\n"
-"   -C                   C, C++, Obj-C or Obj-C++ (default)\n"
+"host language binaries:\n"
+"   ragel, ragel-c       C, C++, Obj-C or Obj-C++\n"
 "                        All code styles supported.\n"
-"   --asm --gas-x86-64-sys-v\n"
-"                        GNU AS, x86_64, System V ABI.\n"
+"   ragel-asm            GNU AS, x86_64, System V ABI.\n"
 "                        Generated in a code style equivalent to -G2\n"
-"   -D                   D           All code styles supported\n"
-"   -Z                   Go          All code styles supported\n"
-"   -A                   C#          -T0 -T1 -F0 -F1 -G0 -G1\n"
-"   -J                   Java        -T0 -T1 -F0 -F1\n"
-"   -R                   Ruby        -T0 -T1 -F0 -F1\n"
-"   -O                   OCaml       -T0 -T1 -F0 -F1\n"
-"   -U                   Rust        -T0 -T1 -F0 -F1\n"
-"   -Y                   Julia       -T0 -T1 -F0 -F1\n"
-"   -K                   Crack       -T0 -T1 -F0 -F1\n"
-"   -P                   JavaScript  -T0 -T1 -F0 -F1\n"
+"   ragel-d              D           All code styles supported\n"
+"   ragel-go             Go          All code styles supported\n"
+"   ragel-csharp         C#          -T0 -T1 -F0 -F1 -G0 -G1\n"
+"   ragel-java           Java        -T0 -T1 -F0 -F1\n"
+"   ragel-ruby           Ruby        -T0 -T1 -F0 -F1\n"
+"   ragel-ocaml          OCaml       -T0 -T1 -F0 -F1\n"
+"   ragel-rust           Rust        -T0 -T1 -F0 -F1\n"
+"   ragel-julia          Julia       -T0 -T1 -F0 -F1\n"
+"   ragel-crack          Crack       -T0 -T1 -F0 -F1\n"
+"   ragel-js             JavaScript  -T0 -T1 -F0 -F1\n"
 "line directives:\n"
 "   -L                   Inhibit writing of #line directives\n"
 "code style:\n"
@@ -639,7 +647,7 @@ void InputData::usage()
 void InputData::version()
 {
 	info() << "Ragel State Machine Compiler version " VERSION << " " PUBDATE << endl <<
-			"Copyright (c) 2001-2019 by Adrian Thurston" << endl;
+			"Copyright (c) 2001-2021 by Dr. Adrian D. Thurston et al." << endl;
 	abortCompile( 0 );
 }
 
@@ -678,6 +686,136 @@ void escapeLineDirectivePath( std::ostream &out, char *path )
 	}
 }
 
+/* Construct a new parameter checker with for paramSpec. */
+ParamCheck::ParamCheck( const char *paramSpec, int argc, const char **argv )
+:
+	state(noparam),
+	argOffset(0),
+	curArg(0),
+	iCurArg(1),
+	paramSpec(paramSpec), 
+	argc(argc), 
+	argv(argv)
+{
+}
+
+/* Check a single option. Returns the index of the next parameter.  Sets p to
+ * the arg character if valid, 0 otherwise.  Sets parg to the parameter arg if
+ * there is one, NULL otherwise. */
+bool ParamCheck::check()
+{
+	bool requiresParam;
+
+	if ( iCurArg >= argc ) {            /* Off the end of the arg list. */
+		state = noparam;
+		return false;
+	}
+
+	if ( argOffset != 0 && *argOffset == 0 ) {
+		/* We are at the end of an arg string. */
+		iCurArg += 1;
+		if ( iCurArg >= argc ) {
+			state = noparam;
+			return false;
+		}
+		argOffset = 0;
+	}
+
+	if ( argOffset == 0 ) {
+		/* Set the current arg. */
+		curArg = argv[iCurArg];
+
+		/* We are at the beginning of an arg string. */
+		if ( argv[iCurArg] == 0 ||        /* Argv[iCurArg] is null. */
+			 argv[iCurArg][0] != '-' ||   /* Not a param. */
+			 argv[iCurArg][1] == 0 ) {    /* Only a dash. */
+			parameter = 0;
+			paramArg = 0;
+
+			iCurArg += 1;
+			state = noparam;
+			return true;
+		}
+		argOffset = argv[iCurArg] + 1;
+	}
+
+	/* Get the arg char. */
+	char argChar = *argOffset;
+	
+	/* Loop over all the parms and look for a match. */
+	const char *pSpec = paramSpec;
+	while ( *pSpec != 0 ) {
+		char pSpecChar = *pSpec;
+
+		/* If there is a ':' following the char then
+		 * it requires a parm.  If a parm is required
+		 * then move ahead two in the parmspec. Otherwise
+		 * move ahead one in the parm spec. */
+		if ( pSpec[1] == ':' ) {
+			requiresParam = true;
+			pSpec += 2;
+		}
+		else {
+			requiresParam = false;
+			pSpec += 1;
+		}
+
+		/* Do we have a match. */
+		if ( argChar == pSpecChar ) {
+			if ( requiresParam ) {
+				if ( argOffset[1] == 0 ) {
+					/* The param must follow. */
+					if ( iCurArg + 1 == argc ) {
+						/* We are the last arg so there
+						 * cannot be a parameter to it. */
+						parameter = argChar;
+						paramArg = 0;
+						iCurArg += 1;
+						argOffset = 0;
+						state = invalid;
+						return true;
+					}
+					else {
+						/* the parameter to the arg is the next arg. */
+						parameter = pSpecChar;
+						paramArg = argv[iCurArg + 1];
+						iCurArg += 2;
+						argOffset = 0;
+						state = match;
+						return true;
+					}
+				}
+				else {
+					/* The param for the arg is built in. */
+					parameter = pSpecChar;
+					paramArg = argOffset + 1;
+					iCurArg += 1;
+					argOffset = 0;
+					state = match;
+					return true;
+				}
+			}
+			else {
+				/* Good, we matched the parm and no
+				 * arg is required. */
+				parameter = pSpecChar;
+				paramArg = 0;
+				argOffset += 1;
+				state = match;
+				return true;
+			}
+		}
+	}
+
+	/* We did not find a match. Bad Argument. */
+	parameter = argChar;
+	paramArg = 0;
+	argOffset += 1;
+	state = invalid;
+	return true;
+}
+
+
 void InputData::parseArgs( int argc, const char **argv )
 {
 	ParamCheck pc( "o:dnmleabjkS:M:I:vHh?-:sT:F:W:G:LpV", argc, argv );
@@ -870,6 +1008,8 @@ void InputData::parseArgs( int argc, const char **argv )
 					histogramFn = strdup(eq);
 				else if ( strcmp( arg, "var-backend" ) == 0 )
 					forceVar = true;
+				else if ( strcmp( arg, "no-fork" ) == 0 )
+					noFork = true;
 				else {
 					error() << "--" << pc.paramArg << 
 							" is an invalid argument" << endl;
@@ -1063,6 +1203,51 @@ char *InputData::readInput( const char *inputFileName )
 	return input;
 }
 
+int InputData::runFrontend( int argc, const char **argv )
+{
+	if ( !process() )
+		return -1;
+	return 0;
+}
+
+int InputData::runRlhc( int argc, const char **argv )
+{
+	struct colm_program *prg;
+	int exit_status;
+
+	prg = colm_new_program( rlhcSections );
+	colm_set_debug( prg, 0 );
+	colm_run_program( prg, argc, argv );
+	exit_status = colm_delete_program( prg );
+	return exit_status;
+}
+
+/* Run a job (frontend or backend). If we want forks then we return the result
+ * via the process's exit code. otherwise it comes back on the stack. */
+int InputData::runJob( const char *what, IdProcess idProcess, int argc, const char **argv )
+{
+#if defined(HAVE_SYS_WAIT_H)
+	if ( !noFork ) {
+		pid_t pid = fork();
+
+		if ( pid == 0 ) {
+			int es = (this->*idProcess)( argc, argv );
+			exit( es );
+		}
+
+		int status = 0;
+		waitpid( pid, &status, 0 );
+		if ( WIFSIGNALED(status) ) {
+			error() << what << " stopped by signal: " << WTERMSIG(status) << std::endl;
+			return -1;
+		}
+
+		return WEXITSTATUS( status );
+	}
+#endif
+	return (this->*idProcess)( argc, argv );
+}
+
 int InputData::main( int argc, const char **argv )
 {
 	int code = 0;
@@ -1082,54 +1267,29 @@ int InputData::main( int argc, const char **argv )
 	return code;
 }
 
-int InputData::rlhcRun( int argc, const char **argv )
-{
-	struct colm_program *prg;
-	int exit_status;
-
-	prg = colm_new_program( rlhcSections );
-	colm_set_debug( prg, 0 );
-	colm_run_program( prg, argc, argv );
-	exit_status = colm_delete_program( prg );
-	return exit_status;
-}
-
-void InputData::wait( const char *what, pid_t pid )
-{
-	int status = 0;
-	waitpid( pid, &status, 0 );
-	if ( WIFSIGNALED(status) )
-		error() << what << " stopped by signal: " << WTERMSIG(status) << std::endl;
-}
-
 int InputData::rlhcMain( int argc, const char **argv )
 {
-	pid_t pid = 0;
+	int code = 0;
+	try {
+		parseArgs( argc, argv );
+		checkArgs();
+		makeDefaultFileName();
+		makeTranslateOutputFileName();
 
-	parseArgs( argc, argv );
-	checkArgs();
-	makeDefaultFileName();
-	makeTranslateOutputFileName();
+		int es = runJob( "frontend", &InputData::runFrontend, 0, 0 );
 
-	pid = fork();
-	if ( pid == 0 ) {
-		/* Child. */
-		if ( !process() )
-			abortCompile( 1 );
-		exit( 0 );
-	}
+		if ( es != 0 )
+			return es;
 
-	wait( "frontend", pid );
-
-	pid = fork();
-	if ( pid == 0 ) {
 		/* rlhc <input> <output> */
 		const char *_argv[] = { "rlhc",
-			genOutputFileName.c_str(),
-			origOutputFileName.c_str(), 0 };
-		rlhcRun( 3, _argv );
-	}
+				genOutputFileName.c_str(),
+				origOutputFileName.c_str(), 0 };
 
-	wait( "rlhc", pid );
-	return 0;
+		return runJob( "rlhc", &InputData::runRlhc, 3, _argv );
+	}
+	catch ( const AbortCompile &ac ) {
+		code = ac.code;
+	}
+	return code;
 }
