@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 Adrian Thurston <thurston@colm.net>
+ * Copyright 2013-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -23,7 +23,7 @@
 #ifndef _LOAD_H
 #define _LOAD_H
 
-#include "ragel.h"
+#include <libfsm/ragel.h>
 
 struct LoadRagel;
 struct InputData;
