@@ -26,10 +26,11 @@
 #include <errno.h>
 #include <limits.h>
 #include <stdlib.h>
-#include <inputdata.h>
+#include "inputdata.h"
 
 /* Parsing. */
-#include "ragel.h"
+#include <libfsm/ragel.h>
+#include <libfsm/action.h>
 #include "parsetree.h"
 #include "parsedata.h"
 
@@ -491,7 +492,7 @@ FsmRes NfaUnion::walk( ParseData *pd )
 		FsmRes res = (*term)->walk( pd );
 		if ( !res.success() ) {
 			/* Delete previos. */
-			for ( int m = 0; m < numMachines; )
+			for ( int m = 0; m < numMachines; ++m)
 				delete machines[m];
 			delete[] machines;
 			return res;
